==> shrunk-1.3.3-py3-none-any.whl: shrunk/util/string.py <==
import re
import datetime


def get_domain(long_url: str) -> str:
    """
    Takes in a url and gets the top level domain,
    e.g. ``https://lmao-d.f.foo.rutgers.edu/lmao.php?thing=true`` becomes ``rutgers.edu``

    :param long_url: The URL from which to extract the domain
    """

    protocol_location = long_url.find('://')
    base_url = long_url[(protocol_location + 3):]  # Strip any protocol
    if protocol_location < 0:
        base_url = long_url

    slash = base_url.find('/')
    domain = base_url[: base_url.find('/')]  # Strip path
    if slash < 0:
        domain = base_url
    # url can contain a-z a hyphen or 0-9 and is seprated by dots.
    # this regex gets rid of any subdomains
    # memes.facebook.com matches facebook.com
    # 1nfo3-384ldnf.doo544-f8.cme-02k4.tk matches cme-02k4.tk
    match = re.search(r'([a-z\-0-9]+\.[a-z\-0-9]+)$', domain, re.IGNORECASE)
    # search for domain if we can't match for a top domain

    return match.group().lower() if match else domain


def formattime(dt: datetime.datetime) -> str:
    """
    Utility function for formatting datetimes.

    :param dt: The datetime to format
    :returns: A string in the format of "Nov 19 2015"
    """
    return dt.strftime('%b %d %Y')


def formatdatetime(dt: datetime.datetime) -> str:
    """
    Utility function for formatting datetimes that may contain both date and time parts.

    :pparam dt: The datetime to format
    :returns: A string in the format of "Sep 01 2020 04:24 PM"
    """
    return dt.strftime('%b %d %Y %I:%M %p')


ip_middle_octet = r'(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5]))'
ip_last_octet = r'(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))'

regex = re.compile(
    r'^'
    # protocol identifier
    r'(?:(?:https?|ftp)://)'
    # user:pass authentication
    r"(?:[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:]+"
    r"(?::[-a-z0-9._~%!$&'()*+,;=:]*)?@)?"
    r'(?:'
    r'(?P<private_ip>'
    # IP address exclusion
    # private & local networks
    r'(?:(?:10|127)' + ip_middle_octet + u'{2}' + ip_last_octet + u')|'
    r'(?:(?:169\.254|192\.168)' + ip_middle_octet + ip_last_octet + u')|'
    r'(?:172\.(?:1[6-9]|2\d|3[0-1])' + ip_middle_octet + ip_last_octet + u'))'
    r'|'
    # private & local hosts
    r'(?P<private_host>'
    r'(?:localhost))'
    r'|'
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    r'(?P<public_ip>'
    r'(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])'
    r'' + ip_middle_octet + u'{2}'
    r'' + ip_last_octet + u')'
    r'|'
    # IPv6 RegEx from https://stackoverflow.com/a/17871737
    r'\[('
    # 1:2:3:4:5:6:7:8
    r'([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|'
    # 1::                              1:2:3:4:5:6:7::
    r'([0-9a-fA-F]{1,4}:){1,7}:|'
    # 1::8             1:2:3:4:5:6::8  1:2:3:4:5:6::8
    r'([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|'
    # 1::7:8           1:2:3:4:5::7:8  1:2:3:4:5::8
    r'([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|'
    # 1::6:7:8         1:2:3:4::6:7:8  1:2:3:4::8
    r'([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|'
    # 1::5:6:7:8       1:2:3::5:6:7:8  1:2:3::8
    r'([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|'
    # 1::4:5:6:7:8     1:2::4:5:6:7:8  1:2::8
    r'([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|'
    # 1::3:4:5:6:7:8   1::3:4:5:6:7:8  1::8
    r'[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|'
    # ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8 ::8       ::
    r':((:[0-9a-fA-F]{1,4}){1,7}|:)|'
    # fe80::7:8%eth0   fe80::7:8%1
    # (link-local IPv6 addresses with zone index)
    r'fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|'
    r'::(ffff(:0{1,4}){0,1}:){0,1}'
    r'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'
    # ::255.255.255.255   ::ffff:255.255.255.255  ::ffff:0:255.255.255.255
    # (IPv4-mapped IPv6 addresses and IPv4-translated addresses)
    r'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|'
    r'([0-9a-fA-F]{1,4}:){1,4}:'
    r'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'
    # 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33
    # (IPv4-Embedded IPv6 Address)
    r'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])'
    r')\]|'
    # host name
    r'(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)'
    # domain name
    r'(?:\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*'
    # TLD identifier
    r'(?:\.(?:[a-z\u00a1-\uffff]{2,}))'
    r')'
    # port number
    r'(?::\d{2,5})?'
    # resource path
    r"(?:/[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:@/]*)?"
    # query string
    r'(?:\?\S*)?'
    # fragment
    r'(?:#\S*)?'
    r'$',
    re.UNICODE | re.IGNORECASE
)

pattern = re.compile(regex)


def validate_url(value: str) -> bool:
    """
    stolen from python-validators

    This validator is based on the wonderful `URL validator of dperini`_.

    .. _URL validator of dperini:
        https://gist.github.com/dperini/729294

    Examples::

        >>> url('http://foobar.dk')
        True

        >>> url('ftp://foobar.dk')
        True

        >>> url('http://10.0.0.1')
        True

        >>> url('http://foobar.d')
        False

    .. versionadded:: 0.2

    .. versionchanged:: 0.10.2

        Added support for various exotic URLs and fixed various false
        positives.

    .. versionchanged:: 0.10.3

        Added ``public`` parameter.

    .. versionchanged:: 0.11.0

        Made the regular expression this function uses case insensitive.

    .. versionchanged:: 0.11.3

        Added support for URLs containing localhost

    .. versionchanged:: 1.0.0

        Removed unused ``public`` parameter.

    .. versionchanged:: 1.3.0

        Return ``False`` instead of raising an exception.

    :param value: URL address string to validate
    :returns: Whether or not the URL was valid
    """

    return pattern.match(value) is not None

==> shrunk-2.3.3-py3-none-any.whl: shrunk/util/string.py <==
import re

__all__ = ['get_domain', 'validate_url']


def get_domain(long_url: str) -> str:
    """
    Takes in a url and gets the top level domain,
    e.g. ``https://lmao-d.f.foo.rutgers.edu/lmao.php?thing=true`` becomes ``rutgers.edu``

    :param long_url: The URL from which to extract the domain
    """

    protocol_location = long_url.find('://')
    base_url = long_url[(protocol_location + 3):]  # Strip any protocol
    if protocol_location < 0:
        base_url = long_url

    slash = base_url.find('/')
    domain = base_url[: base_url.find('/')]  # Strip path
    if slash < 0:
        domain = base_url
    # url can contain a-z a hyphen or 0-9 and is seprated by dots.
    # this regex gets rid of any subdomains
    # memes.facebook.com matches facebook.com
    # 1nfo3-384ldnf.doo544-f8.cme-02k4.tk matches cme-02k4.tk
    match = re.search(r'([a-z\-0-9]+\.[a-z\-0-9]+)$', domain, re.IGNORECASE)
    # search for domain if we can't match for a top domain

    return match.group().lower() if match else domain


ip_middle_octet = r'(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5]))'
ip_last_octet = r'(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))'

regex = re.compile(
    r'^'
    # protocol identifier
    r'(?:(?:https?|ftp)://)'
    # user:pass authentication
    r"(?:[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:]+"
    r"(?::[-a-z0-9._~%!$&'()*+,;=:]*)?@)?"
    r'(?:'
    r'(?P<private_ip>'
    # IP address exclusion
    # private & local networks
    r'(?:(?:10|127)' + ip_middle_octet + u'{2}' + ip_last_octet + u')|'
    r'(?:(?:169\.254|192\.168)' + ip_middle_octet + ip_last_octet + u')|'
    r'(?:172\.(?:1[6-9]|2\d|3[0-1])' + ip_middle_octet + ip_last_octet + u'))'
    r'|'
    # private & local hosts
    r'(?P<private_host>'
    r'(?:localhost))'
    r'|'
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    r'(?P<public_ip>'
    r'(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])'
    r'' + ip_middle_octet + u'{2}'
    r'' + ip_last_octet + u')'
    r'|'
    # IPv6 RegEx from https://stackoverflow.com/a/17871737
    r'\[('
    # 1:2:3:4:5:6:7:8
    r'([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|'
    # 1::                              1:2:3:4:5:6:7::
    r'([0-9a-fA-F]{1,4}:){1,7}:|'
    # 1::8             1:2:3:4:5:6::8  1:2:3:4:5:6::8
    r'([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|'
    # 1::7:8           1:2:3:4:5::7:8  1:2:3:4:5::8
    r'([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|'
    # 1::6:7:8         1:2:3:4::6:7:8  1:2:3:4::8
    r'([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|'
    # 1::5:6:7:8       1:2:3::5:6:7:8  1:2:3::8
    r'([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|'
    # 1::4:5:6:7:8     1:2::4:5:6:7:8  1:2::8
    r'([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|'
    # 1::3:4:5:6:7:8   1::3:4:5:6:7:8  1::8
    r'[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|'
    # ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8 ::8       ::
    r':((:[0-9a-fA-F]{1,4}){1,7}|:)|'
    # fe80::7:8%eth0   fe80::7:8%1
    # (link-local IPv6 addresses with zone index)
    r'fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|'
    r'::(ffff(:0{1,4}){0,1}:){0,1}'
    r'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'
    # ::255.255.255.255   ::ffff:255.255.255.255  ::ffff:0:255.255.255.255
    # (IPv4-mapped IPv6 addresses and IPv4-translated addresses)
    r'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|'
    r'([0-9a-fA-F]{1,4}:){1,4}:'
    r'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'
    # 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33
    # (IPv4-Embedded IPv6 Address)
    r'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])'
    r')\]|'
    # host name
    r'(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)'
    # domain name
    r'(?:\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*'
    # TLD identifier
    r'(?:\.(?:[a-z\u00a1-\uffff]{2,}))'
    r')'
    # port number
    r'(?::\d{2,5})?'
    # resource path
    r"(?:/[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:@/]*)?"
    # query string
    r'(?:\?\S*)?'
    # fragment
    r'(?:#\S*)?'
    r'$',
    re.UNICODE | re.IGNORECASE,
)

pattern = re.compile(regex)


def validate_url(value: str) -> bool:
    """
    stolen from python-validators

    This validator is based on the wonderful `URL validator of dperini`_.

    .. _URL validator of dperini:
        https://gist.github.com/dperini/729294

    Examples::

        >>> url('http://foobar.dk')
        True

        >>> url('ftp://foobar.dk')
        True

        >>> url('http://10.0.0.1')
        True

        >>> url('http://foobar.d')
        False

    .. versionadded:: 0.2

    .. versionchanged:: 0.10.2

        Added support for various exotic URLs and fixed various false
        positives.

    .. versionchanged:: 0.10.3

        Added ``public`` parameter.

    .. versionchanged:: 0.11.0

        Made the regular expression this function uses case insensitive.

    .. versionchanged:: 0.11.3

        Added support for URLs containing localhost

    .. versionchanged:: 1.0.0

        Removed unused ``public`` parameter.

    .. versionchanged:: 1.3.0

        Return ``False`` instead of raising an exception.

    :param value: URL address string to validate
    :returns: Whether or not the URL was valid
    """

    return pattern.match(value) is not None
